@@ -45,12 +45,19 @@ def main(argv=None):
             print('{} = {}'.format(key, value))
         return 0
 
+    # FIXME: default values are reporting incorrectly or are missformated
+    class RawDescriptionDefaultsHelpFormatter(
+            argparse.RawDescriptionHelpFormatter,
+            argparse.ArgumentDefaultsHelpFormatter):
+        pass
+
     parser = argparse.ArgumentParser(
         prog='xdoctest',
         description=(
             'Xdoctest {xdoc_version} - on Python - {sys_version} - '
             'discover and run doctests within a python package'
-        ).format(**version_info)
+        ).format(**version_info),
+        formatter_class=RawDescriptionDefaultsHelpFormatter,
     )
     parser.add_argument(
         'arg', nargs='*', help=utils.codeblock(
@@ -65,7 +72,7 @@ def main(argv=None):
     from xdoctest import doctest_example
     from xdoctest import runner
     runner._update_argparse_cli(parser.add_argument)
-    doctest_example.Config()._update_argparse_cli(parser.add_argument)
+    doctest_example.DoctestConfig()._update_argparse_cli(parser.add_argument)
 
     args, unknown = parser.parse_known_args(args=argv[1:])
     ns = args.__dict__.copy()
@@ -127,7 +134,7 @@ def main(argv=None):
         ns['options'] = options
 
     from xdoctest import doctest_example
-    config = doctest_example.Config()._populate_from_cli(ns)
+    config = doctest_example.DoctestConfig()._populate_from_cli(ns)
 
     import textwrap
     if config['verbose'] > 2:
