@@ -20,8 +20,8 @@ Advanced Directives
 
 Advanced directives may take arguments, be conditional, or modify the runtime
 state in complex ways.  For instance, whereas most directives modify a boolean
-value in the runtime state, the advanced `REQUIRES` directive either adds or
-removes a value from a `set` of unmet requirements. Doctests will only run if
+value in the runtime state, the advanced ``REQUIRES`` directive either adds or
+removes a value from a ``set`` of unmet requirements. Doctests will only run if
 there are no unmet requirements.
 
 
@@ -30,7 +30,7 @@ CommandLine:
 
 
 Example:
-    The following example shows how the `+SKIP` directives may be used to
+    The following example shows how the ``+SKIP`` directives may be used to
     bypass certain places in the code.
 
     >>> # An inline directive appears on the same line as a command and
@@ -57,7 +57,7 @@ Example:
     >>> print('This line will print: (F)')
 
 Example:
-    This next examples illustrates how to use the advanced `+REQURIES()`
+    This next examples illustrates how to use the advanced ``+REQURIES()``
     directive. Note, the REQUIRES and SKIP states are independent.
 
     >>> import sys
@@ -88,6 +88,7 @@ import os
 import re
 import copy
 import warnings
+import operator
 from xdoctest import static_analysis as static
 from xdoctest import utils
 from collections import OrderedDict
@@ -143,7 +144,7 @@ Effect = namedtuple('Effect', ('action', 'key', 'value'))
 
 class RuntimeState(utils.NiceRepr):
     """
-    Maintains the runtime state for a single `run()` of an example
+    Maintains the runtime state for a single ``run()`` of an example
 
     Inline directives are pushed and popped after the line is run.
     Otherwise directives persist until another directive disables it.
@@ -237,38 +238,40 @@ class RuntimeState(utils.NiceRepr):
         Update the runtime state given a set of directives
 
         Args:
-            directives (List[Directive]): list of directives. The `effect`
+            directives (List[Directive]): list of directives. The ``effects``
                 method is used to update this object.
         """
         # Clear the previous inline state
         self._inline_state.clear()
         for directive in directives:
+            for effect in directive.effects():
+                action, key, value = effect
+                if action == 'noop':
+                    continue
 
-            action, key, value = directive.effect()
+                if key not in self._global_state:
+                    warnings.warn('Unknown state: {}'.format(key))
 
-            if action == 'noop':
-                continue
+                # Determine if this impacts the local (inline) or global state.
+                if directive.inline:
+                    state = self._inline_state
+                else:
+                    state = self._global_state
 
-            if key not in self._global_state:
-                warnings.warn('Unknown state: {}'.format(key))
-
-            # Determine if this impacts the local (inline) or global state.
-            if directive.inline:
-                state = self._inline_state
-            else:
-                state = self._global_state
-
-            if action == 'set_report_style':
-                # Special handling of report style
-                self.set_report_style(key.replace('REPORT_', ''))
-            elif action == 'assign':
-                state[key] = value
-            elif action == 'set.add':
-                state[key].add(value)
-            elif action == 'set.remove':
-                state[key].remove(value)
-            else:
-                raise KeyError('unknown action {}'.format(action))
+                if action == 'set_report_style':
+                    # Special handling of report style
+                    self.set_report_style(key.replace('REPORT_', ''))
+                elif action == 'assign':
+                    state[key] = value
+                elif action == 'set.add':
+                    state[key].add(value)
+                elif action == 'set.remove':
+                    try:
+                        state[key].remove(value)
+                    except KeyError:
+                        pass
+                else:
+                    raise KeyError('unknown action {}'.format(action))
 
 
 class Directive(utils.NiceRepr):
@@ -294,8 +297,8 @@ class Directive(utils.NiceRepr):
             Directive: directive: the parsed directives
 
         Notes:
-            The original `doctest` module sometimes yeilded false positives for a
-            directive pattern. Because `xdoctest` is parsing the text, this issue
+            The original ``doctest`` module sometimes yeilded false positives for a
+            directive pattern. Because ``xdoctest`` is parsing the text, this issue
             does not occur.
 
         Example:
@@ -322,6 +325,11 @@ class Directive(utils.NiceRepr):
             >>> print(', '.join(list(map(str, Directive.extract(text)))))
             <Directive(+ELLIPSIS)>, <Directive(-NORMALIZE_WHITESPACE)>
 
+            >>> # Make sure commas inside parens are not split
+            >>> text = '# xdoctest: +REQUIRES(module:foo,module:bar)'
+            >>> print(', '.join(list(map(str, Directive.extract(text)))))
+            <Directive(+REQUIRES(module:foo, module:bar))>
+
         Example:
             >>> any(Directive.extract(' # xdoctest: skip'))
             True
@@ -343,7 +351,9 @@ class Directive(utils.NiceRepr):
             if m:
                 for key, optstr in m.groupdict().items():
                     if optstr:
-                        for optpart in optstr.split(','):
+                        optparts = _split_opstr(optstr)
+                        # optparts = optstr.split(',')
+                        for optpart in optparts:
                             directive = parse_directive_optstr(optpart, inline)
                             if directive:
                                 yield directive
@@ -357,6 +367,7 @@ class Directive(utils.NiceRepr):
             return '{}{}'.format(prefix, self.name)
 
     def _unpack_args(self, num):
+        warnings.warning('Deprecated and will be removed', DeprecationWarning)
         nargs = self.args
         if len(nargs) != 1:
             raise TypeError(
@@ -364,86 +375,156 @@ class Directive(utils.NiceRepr):
                 'got {}'.format(self.name, num, nargs))
         return self.args
 
-    def effect(self, argv=None):
+    def effect(self, argv=None, environ=None):
+        warnings.warning('Deprecated use effects', DeprecationWarning)
+        effects = self.effects(argv=argv, environ=environ)
+        if len(effects) > 1:
+            raise Exception('Old method cannot hanldle multiple effects')
+        return effects[0]
+
+    def effects(self, argv=None, environ=None):
         """
         Returns how this directive modifies a RuntimeState object
 
-        This is used by a RuntimeState object to update itself
+        This is called by :func:`RuntimeState.update` to update itself
+
+        Args:
+            argv (List[str], default=None):
+                if specified, overwrite sys.argv
+            environ (Dict[str, str], default=None):
+                if specified, overwrite os.environ
 
         Returns:
-            Effect: named tuple containing:
+            List[Effect]: list of named tuples containing:
                 action (str): code indicating how to update
                 key (str): name of runtime state item to modify
                 value (object): value to modify with
 
         CommandLine:
-            xdoctest -m xdoctest.directive Directive.effect
+            xdoctest -m xdoctest.directive Directive.effects
 
         Example:
-            >>> Directive('SKIP').effect()
+            >>> Directive('SKIP').effects()[0]
             Effect(action='assign', key='SKIP', value=True)
-            >>> Directive('SKIP', inline=True).effect()
+            >>> Directive('SKIP', inline=True).effects()[0]
             Effect(action='assign', key='SKIP', value=True)
-            >>> Directive('REQUIRES', args=['-s']).effect(argv=['-s'])
-            Effect(action='noop', key='REQUIRES', value=None)
-            >>> Directive('REQUIRES', args=['-s']).effect(argv=[])
+            >>> Directive('REQUIRES', args=['-s']).effects(argv=['-s'])[0]
+            Effect(action='noop', key='REQUIRES', value='-s')
+            >>> Directive('REQUIRES', args=['-s']).effects(argv=[])[0]
             Effect(action='set.add', key='REQUIRES', value='-s')
-            >>> Directive('ELLIPSIS', args=['-s']).effect(argv=[])
+            >>> Directive('ELLIPSIS', args=['-s']).effects(argv=[])[0]
             Effect(action='assign', key='ELLIPSIS', value=True)
 
         Doctest:
             >>> # requirement directive with module
             >>> directive = list(Directive.extract('# xdoctest: requires(module:xdoctest)'))[0]
             >>> print('directive = {}'.format(directive))
-            >>> print('directive.effect() = {}'.format(directive.effect()))
+            >>> print('directive.effects() = {}'.format(directive.effects()[0]))
             directive = <Directive(+REQUIRES(module:xdoctest))>
-            directive.effect() = Effect(action='noop', key='REQUIRES', value=None)
+            directive.effects() = Effect(action='noop', key='REQUIRES', value='module:xdoctest')
 
             >>> directive = list(Directive.extract('# xdoctest: requires(module:notamodule)'))[0]
             >>> print('directive = {}'.format(directive))
-            >>> print('directive.effect() = {}'.format(directive.effect()))
+            >>> print('directive.effects() = {}'.format(directive.effects()[0]))
             directive = <Directive(+REQUIRES(module:notamodule))>
-            directive.effect() = Effect(action='set.add', key='REQUIRES', value='module:notamodule')
+            directive.effects() = Effect(action='set.add', key='REQUIRES', value='module:notamodule')
+
+            >>> directive = list(Directive.extract('# xdoctest: requires(env:FOO==1)'))[0]
+            >>> print('directive = {}'.format(directive))
+            >>> print('directive.effects() = {}'.format(directive.effects(environ={})[0]))
+            directive = <Directive(+REQUIRES(env:FOO==1))>
+            directive.effects() = Effect(action='set.add', key='REQUIRES', value='env:FOO==1')
+
+            >>> directive = list(Directive.extract('# xdoctest: requires(env:FOO==1)'))[0]
+            >>> print('directive = {}'.format(directive))
+            >>> print('directive.effects() = {}'.format(directive.effects(environ={'FOO': '1'})[0]))
+            directive = <Directive(+REQUIRES(env:FOO==1))>
+            directive.effects() = Effect(action='noop', key='REQUIRES', value='env:FOO==1')
+
+            >>> # requirement directive with two args
+            >>> directive = list(Directive.extract('# xdoctest: requires(--show, module:xdoctest)'))[0]
+            >>> print('directive = {}'.format(directive))
+            >>> for effect in directive.effects():
+            >>>     print('effect = {!r}'.format(effect))
+            directive = <Directive(+REQUIRES(--show, module:xdoctest))>
+            effect = Effect(action='set.add', key='REQUIRES', value='--show')
+            effect = Effect(action='noop', key='REQUIRES', value='module:xdoctest')
         """
         key = self.name
         value = None
 
+        effects = []
         if self.name == 'REQUIRES':
             # Special handling of REQUIRES
-            if argv is None:
-                argv = sys.argv
-            arg, = self._unpack_args(1)
-            if _is_requires_satisfied(arg, argv):
-                # If the requirement is met, then do nothing,
-                action = 'noop'
-            else:
-                # otherwise, add or remove the condtion from REQUIREMENTS,
-                # depending on if the directive is positive or negative.
+            for arg in self.args:
                 value = arg
-                if self.positive:
-                    action = 'set.add'
+                if _is_requires_satisfied(arg, argv=argv, environ=environ):
+                    # If the requirement is met, then do nothing,
+                    action = 'noop'
                 else:
-                    action = 'set.remove'
+                    # otherwise, add or remove the condtion from REQUIREMENTS,
+                    # depending on if the directive is positive or negative.
+                    if self.positive:
+                        action = 'set.add'
+                    else:
+                        action = 'set.remove'
+                effects.append(Effect(action, key, value))
         elif key.startswith('REPORT_'):
             # Special handling of report style
             if self.positive:
                 action = 'noop'
             else:
                 action = 'set_report_style'
+            effects.append(Effect(action, key, value))
         else:
             # The action overwrites state[key] using value
             action = 'assign'
             value = self.positive
-        return Effect(action, key, value)
+            effects.append(Effect(action, key, value))
+        return effects
 
 
-def _is_requires_satisfied(arg, argv):
+def _split_opstr(optstr):
+    """
+    Simplified balanced paren logic to only split commas outside of parens
+
+    Example:
+        >>> optstr = '+FOO, REQUIRES(foo,bar), +ELLIPSIS'
+        >>> _split_opstr(optstr)
+        ['+FOO', 'REQUIRES(foo,bar)', '+ELLIPSIS']
+    """
+    import re
+    stack = []
+    split_pos = []
+    for match in re.finditer(r',|\(|\)', optstr):
+        token = match.group()
+        if token == ',' and not stack:
+            # Only split when there are no parens
+            split_pos.append(match.start())
+        elif token == '(':
+            stack.append(token)
+        elif token == ')':
+            stack.pop()
+    assert len(stack) == 0, 'parens not balanced'
+
+    parts = []
+    prev = 0
+    for curr in split_pos:
+        parts.append(optstr[prev:curr].strip())
+        prev = curr + 1
+    curr = None
+    parts.append(optstr[prev:curr].strip())
+    return parts
+
+
+def _is_requires_satisfied(arg, argv=None, environ=None):
     """
     Determines if the argument to a REQUIRES directive is satisfied
 
     Args:
         arg (str): condition code
-        argv (List[str]): cmdline if arg is cmd code
+        argv (List[str]): cmdline if arg is cmd code usually ``sys.argv``
+        environ (Dict[str, str]): environment variables usually ``os.environ``
 
     Returns:
         bool: flag - True if the requirement is met
@@ -455,6 +536,32 @@ def _is_requires_satisfied(arg, argv):
         >>> _is_requires_satisfied('pypy', argv=[])
         >>> _is_requires_satisfied('nt', argv=[])
         >>> _is_requires_satisfied('linux', argv=[])
+
+        >>> _is_requires_satisfied('env:FOO', argv=[], environ={'FOO': '1'})
+        True
+        >>> _is_requires_satisfied('env:FOO==1', argv=[], environ={'FOO': '1'})
+        True
+        >>> _is_requires_satisfied('env:FOO==T', argv=[], environ={'FOO': '1'})
+        False
+        >>> _is_requires_satisfied('env:BAR', argv=[], environ={'FOO': '1'})
+        False
+        >>> _is_requires_satisfied('env:BAR==1', argv=[], environ={'FOO': '1'})
+        False
+        >>> _is_requires_satisfied('env:BAR!=1', argv=[], environ={'FOO': '1'})
+        True
+        >>> _is_requires_satisfied('env:BAR!=1', argv=[], environ={'BAR': '0'})
+        True
+        >>> _is_requires_satisfied('env:BAR!=1')
+        ...
+
+        >>> # xdoctest: +REQUIRES(module:pytest)
+        >>> import pytest
+        >>> with pytest.raises(ValueError):
+        >>>     _is_requires_satisfied('badflag:BAR==1', [])
+
+        >>> import pytest
+        >>> with pytest.raises(KeyError):
+        >>>     _is_requires_satisfied('env:BAR>=1', argv=[], environ={'BAR': '0'})
     """
     # TODO: add python version options
     SYS_PLATFORM_TAGS = ['win32', 'linux', 'darwin', 'cywgin']
@@ -463,28 +570,57 @@ def _is_requires_satisfied(arg, argv):
     # TODO: tox tags: https://tox.readthedocs.io/en/latest/example/basic.html
     PY_VER_TAGS = ['py2', 'py3']
 
+    arg_lower = arg.lower()
+
     if arg.startswith('-'):
+        if argv is None:
+            argv = sys.argv
         flag = arg in argv
     elif arg.startswith('module:'):
         parts = arg.split(':')
         if len(parts) != 2:
-            raise ValueError('xdoctest REQUIRES directive has too many parts')
+            raise ValueError('xdoctest module REQUIRES directive has too many parts')
         # set flag to False (aka SKIP) if the module does not exist
         modname = parts[1]
         flag = _module_exists(modname)
-    elif arg.lower() in SYS_PLATFORM_TAGS:
-        flag = sys.platform.startswith(arg.lower())
-    elif arg.lower() in OS_NAME_TAGS:
-        flag = os.name.startswith(arg.lower())
-    elif arg.lower() in PY_IMPL_TAGS:
-        import platform
-        flag = platform.python_implementation().startswith(arg.lower())
-    elif arg.lower() in PY_VER_TAGS:
-        if sys.version_info[0] == 2:  # nocover
-            flag = arg.lower() == 'PY2'
-        elif sys.version_info[0] == 3:
-            flag = arg.lower() == 'PY3'
+    elif arg.startswith('env:'):
+        if environ is None:
+            environ = os.environ
+        parts = arg.split(':')
+        if len(parts) != 2:
+            raise ValueError('xdoctest env REQUIRES directive has too many parts')
+        envexpr = parts[1]
+        expr_parts = re.split('(==|!=|>=)', envexpr)
+        if len(expr_parts) == 1:
+            # Test if the environment variable is truthy
+            env_key = expr_parts[0]
+            flag = bool(environ.get(env_key, None))
+        elif len(expr_parts) == 3:
+            # Test if the environment variable is equal to an expression
+            env_key, op_code, value = expr_parts
+            env_val = environ.get(env_key, None)
+            if op_code == '==':
+                op = operator.eq
+            elif op_code == '!=':
+                op = operator.ne
+            else:
+                raise KeyError(op_code)
+            flag = op(env_val, value)
         else:
+            raise ValueError('Too many expr_parts={}'.format(expr_parts))
+    elif arg_lower in SYS_PLATFORM_TAGS:
+        flag = sys.platform.startswith(arg_lower)
+    elif arg_lower in OS_NAME_TAGS:
+        flag = os.name.startswith(arg_lower)
+    elif arg_lower in PY_IMPL_TAGS:
+        import platform
+        flag = platform.python_implementation().startswith(arg_lower)
+    elif arg_lower in PY_VER_TAGS:
+        if sys.version_info[0] == 2:  # nocover
+            flag = arg_lower == 'py2'
+        elif sys.version_info[0] == 3:  # pragma: nobranch
+            flag = arg_lower == 'py3'
+        else:  # nocover
             flag = False
     else:
         msg = utils.codeblock(
@@ -493,6 +629,7 @@ def _is_requires_satisfied(arg, argv):
             (1) a PLATFORM or OS tag (e.g. win32, darwin, linux),
             (2) a command line flag prefixed with '--', or
             (3) a module prefixed with 'module:'.
+            (4) an environment variable prefixed with 'env:'.
             Got arg={!r}
             ''').replace('\n', ' ').strip().format(arg)
         raise ValueError(msg)
@@ -529,7 +666,7 @@ def parse_directive_optstr(optpart, inline=None):
     Parses the information in the directive from the "optpart"
 
     optstrs are:
-        optionally prefixed with `+` (default) or `-`
+        optionally prefixed with ``+`` (default) or ``-``
         comma separated
         may contain one paren enclosed argument (experimental)
         all spaces are ignored
@@ -548,8 +685,9 @@ def parse_directive_optstr(optpart, inline=None):
     paren_pos = optpart.find('(')
     if paren_pos > -1:
         # handle simple paren case.
-        # TODO expand or remove
-        args = [optpart[paren_pos + 1:optpart.find(')')]]
+        body = optpart[paren_pos + 1:optpart.find(')')]
+        args = [a.strip() for a in body.split(',')]
+        # args = [optpart[paren_pos + 1:optpart.find(')')]]
         optpart = optpart[:paren_pos]
     else:
         args = []
