@@ -104,7 +104,7 @@ def pytest_addoption(parser):
                     dest='xdoctest_analysis')
 
     from xdoctest import doctest_example
-    doctest_example.Config()._update_argparse_cli(
+    doctest_example.DoctestConfig()._update_argparse_cli(
         group.addoption, prefix=['xdoctest', 'xdoc'],
         defaults=dict(verbose=0)
     )
@@ -153,6 +153,7 @@ class ReprFailXDoctest(code.TerminalRepr):
 class XDoctestItem(pytest.Item):
     def __init__(self, name, parent, example=None):
         super(XDoctestItem, self).__init__(name, parent)
+        self.cls = XDoctestItem
         self.example = example
         self.obj = None
         self.fixture_request = None
@@ -205,7 +206,7 @@ class _XDoctestBase(pytest.Module):
         ns = NamespaceLike(self.config)
 
         from xdoctest import doctest_example
-        self._examp_conf = doctest_example.Config()._populate_from_cli(ns)
+        self._examp_conf = doctest_example.DoctestConfig()._populate_from_cli(ns)
 
 
 class XDoctestTextfile(_XDoctestBase):
